/***************************************************************************\

  File........: avr_usart.h
  Author(s)...: Gerhard Brnner
  Target(s)...: AVR_MEGA8
  Compiler....: WIN_AVR
  Description.: Routines for the internal hardware USART.
  Header-File fr AVR_usart.c

  Revisions...: V2.1
  2009.11.11  - removed type byte_t
  2009.02.01  - added sub-functions for receive-buffer and receive-interrupt
  2008.07.04  - added request for defined USART0_RX_vect ...

\***************************************************************************/

#ifndef AVR_USART_H
#define AVR_USART_H

/* Include-Dateien
****************************************************************************/
#include <avr\io.h> /* richtige Header-Datei wird entsprechend der Einstellungen der Makedatei eingezogen*/
#include <avr\interrupt.h>
#include <stdio.h>

/* Sende-/Empfangspuffer
****************************************************************************/
	#define _MAX_BUFFER_LEN		32		/* Modulo 2 Groesse: nur 2,4,8,16,32 Bytes */


/* Baudrate - Prescaler calculation
****************************************************************************/
	#ifndef _XTAL
		#define _XTAL		F_CPU+'UL'
		/* Taktfrequenz des Prozessors in Hz
		   >> Vorsicht mus unsigned long sein sonst kommt es zu fehlerhaften Baudraten Berechnung */
	#endif
	/* Hilfsmakro zur UBRR-Berechnung ("Formel" laut Datenblatt) fr Single-Speed mode*/
	#define UART_UBRR_CALC(BAUD_) (((_XTAL)/((BAUD_)*16L))-1)


/* Typdefinitionen
****************************************************************************/
	typedef uint8_t uint8_t;

#ifdef AVR_USART_C
	typedef struct{
				uint8_t Data[_MAX_BUFFER_LEN];
                uint16_t WritePos; 			/* Gibt akt. Schreibposition an */
				uint16_t ReadPos;  			/* Gibt akt. Leseposition an */
                uint8_t Overflow_cnt; 		/* Counter fr berlauf des Puffers */
				} ringbuffer_t;
#endif

/* Funktions-Prototypen
****************************************************************************/
	void 	init_usart_9600_8_1_N_Asyn(void);
	void 	Usart_puts (char *s);
	void 	Usart_putc(uint8_t data);
	uint8_t Usart_getc(void);
	uint8_t usart_read_buffer_string(uint8_t *data);

#endif /* avr_usart */
