/***************************************************************************/
/**
	@file		cmd_interface.h
	@brief		Header-File for cmd_interface.c
	@brief		Simple implementation of an command-interface
	@brief		Target/Compiler: AVR_MEGA / GCC(WinAVR)
	@author		Gerhard Bruenner
	@version	V2.01

	@date		2009.11.15  - added function cmd_interface_add_cmd and Backspace-Function - GB
	@date		2009.10.31  - changed the way how commandline is interpret - GB
	@date		2009.10.21  - small updates   - GB
	@date		2009.02.01  - Created         - GB

	@bug		none that I am aware so far

\***************************************************************************/
#ifndef CMD_INTERFACE_H
#define CMD_INTERFACE_H

/* Include-Dateien
****************************************************************************/
#include <avr\io.h> /* richtige Header-Datei wird entsprechend der Einstellungen der Makedatei eingezogen*/
#include <stdio.h>


/* Defines to PARAMETRISE INTERPRETER
****************************************************************************/
#define _CMD_INTERFACE_MAX_LENGTH			80				/* max number of signs allowed in a command line !! MUST be lower than 127 char*/
#define _CMD_INTERFACE_MAX_OPERAND_LENGTH	12				/* max.number of signs per operand in a command line*/
#define _CMD_INTERFACE_MAX_NO_OPERANDS		5				/* max. no of operands per command line including the command */
#define _CMD_INTERFACE_NO_OF_COMMANDS		5				/* amount of possible commands */
#define _CMD_INTERFACE_OPERAND_SPLIT_SIGN	_ASCII_COMMA	/* sign which splits operands of a cmd-line - ascii 44= komma*/
#define _CMD_INTERFACE_COMMAND_SPLIT_SIGN	_ASCII_SPACE	/* sign which splits command from operands in a cmd-line - ascii 32=Space  */
#define _CMD_INTERFACE_EOL					_ASCII_CR 		/* End of Line - Sign = carriage return */


/* Defines
****************************************************************************/

	/* ASCII - CHARACTERS */
	#define _ASCII_BEL				0x07	/* "Bell"				*/
	#define _ASCII_BS				0x08	/* Back Space 			*/
	#define _ASCII_CR				0x0D	/* Carriage Return 		*/
	#define _ASCII_LF				0x0A	/* Line Feed 			*/
	#define _ASCII_ESC				0x1B	/* Escape				*/
	#define _ASCII_DEL				0x7F	/* Delete				*/
	#define _ASCII_SPACE			0x20	/* Space				*/
	#define _ASCII_COMMA			0x2C	/* Comma				*/

	/* ERROR - Bitcodes of the function execute_cmd_interface */
	#define _CMD_INTERFACE_ERR_MASK			0xF8
	#define _CMD_INTERFACE_ERR_OP_CHAR		0x84
	#define _CMD_INTERFACE_ERR_OP_2_MANY	0x90
	#define _CMD_INTERFACE_ERR_NO_CMD		0xA0
	#define _CMD_INTERFACE_ERR_CMD_LINE		0xB0

/* Typdefinitionen
****************************************************************************/
typedef struct  {
	char 		*cmd;
	uint8_t		(*cmd_func_ptr)	(char *op1, char *op2, char *op3, char *op4);
	} cmd_interface_command_table_t;


/* Variables
****************************************************************************/
#ifndef cmd_interface_c
	extern volatile cmd_interface_command_table_t cmd_interface_table[_CMD_INTERFACE_NO_OF_COMMANDS];
#endif

/* Funktions-Prototypen
****************************************************************************/
int8_t 	execute_cmd_interface(char *string);
void 	cmd_interface_init();
uint8_t cmd_interface_add_cmd(char (*cmd_string), uint8_t (*cmd_func_ptr)() );

#endif /* cmd_interface_h */
