/***************************************************************************/
/**
	@file		string_addon.c
	@brief		some additional clever string functions
	@brief		Target/Compiler: AVR_MEGA / GCC(WinAVR)
	@author		Gerhard Bruenner
	@version	V0.11

	@date		2009.10.31  - Created              - GB

	@bug		none that I am aware so far

\***************************************************************************/


#define   STRING_ADDON_C

/* Include-Dateien
****************************************************************************/
#include "string_addon.h"

/* Internal Defines
****************************************************************************/
 	
/* Function Prototypes
****************************************************************************/

/* globale int Variablen
****************************************************************************/

/* Functions
****************************************************************************/

/**
 	@brief		eleminates all leading signs which are equal to the given one
 	@param		*string - string to investigate
 	@param		sign - character which shall be eliminated
 	@return		void
 	
 	@details    This function eliminates all leading characters in a string.
				The character which shall be eliminated and the string to be 
				checked have to be addressed in the call. 
 */

void r_trim(char *string, char sign) {
	register uint8_t 	a_cnt=0;
	register char 		act_char;
	register uint8_t	b_cnt=0;
	act_char=string[a_cnt];	
	/* search for first character which is not the same like the one in char sign */
	while ((act_char!=0) && (act_char==sign)) 	act_char=string[++a_cnt];
	/* move content */
	while (act_char!=0) {
	 	string[b_cnt++]=act_char;
	  	act_char=string[++a_cnt];
	}
	string[b_cnt]=0;	
}/* void r_trim(char *string, char sign) */


/**
 	@brief		eleminates all signs at the end of a string which are equal to the given one
 	@param		*string - string to investigate
 	@param		sign - character which shall be eliminated
 	@return		void
 	
 	@details    This function eliminates all characters at the end of a string.
				The character which shall be eliminated and the string to be 
				checked have to be addressed in the call. 
 */
void l_trim(char *string, char sign) {
	register uint8_t 	a_cnt=0;
	/* Search for the end of the string */
	while(string[a_cnt]!=0)	a_cnt++;
	/* search at the end for char which is not the same like the one in char sign */
	while (string[--a_cnt]==sign);
	string[++a_cnt]=0;	
}/* void l_trim(char *string, char sign) */


/**
 	@brief		eleminates all signs of the given type at the end or at the beginning of a string
 	@param		*string - string to investigate
 	@param		sign - character which shall be eliminated
 	@return		void
 	
 	@details    This function eliminates all characters at the end and at the beginning of a string.
				The character which shall be eliminated and the string to be 
				checked have to be addressed in the call. 
				------- Testcode
				sprintf(str_rcv_usart,"Gerhard   ");
				trim(str_rcv_usart,' ');
				sprintf(str_rcv_usart," Gerhard ");
				trim(str_rcv_usart,' ');
				sprintf(str_rcv_usart,"Gerhard");
				trim(str_rcv_usart,' ');
 */
void trim(char *string, char sign) {
	register uint8_t 	a_cnt=0;
	register char 		act_char;
	register uint8_t	b_cnt=0;
	act_char=string[a_cnt];	
	/* search for first character which is not the same like the one in char sign */
	while ((act_char!=0) && (act_char==sign)) 	act_char=string[++a_cnt];
	/* move content */
	while (act_char!=0){
	 	string[b_cnt++]=act_char;
	  	act_char=string[++a_cnt];
	}
	/* remove tailing signs */
	do {
		act_char=string[--b_cnt];
	} while (act_char==sign);
	string[++b_cnt]=0;	
}/* void trim(char *string, char sign) */


/**
 	@brief		functions which splits a string into two parts at the position of a certain character
 	@param		char 	*string 			- pointer onto string which shall be split
 	@param		char 	*splitted_string	- part which have been splited from content of *string
 	@param		char 	split_sign	 		- character to be used as split sign
 	@param		uint8_t	max_char2Copy		- 
 	@return		<0 	- error
	@return		-1	- to many signs to copy 
 	@return		 0	- split char has not been found in string
 	@return		>0	- index of *string after the split char found by the routine
 	@details    This functions searches for a character given by the user via the parameter split_sign at
				the beginning of the given string-pointer. In the case the split_sign could not be found, the
				function returns -1. In the case the split_signs has been found, it copys the content of 
				string into the string splitted_string till the point where the split_sign has been found.
				The return-value contains the index of the sign AFTER the split_sign. It is upto the user
				to delete the splitted content in string or to restart the function at the position in the
				return-value
				--------- example:		
				sprintf( str_rcv_usart,"Gerhard ,12345,macht was   ,er  will");
				iCnt = split_string(&str_rcv_usart[0],&spliter_string[0],',',20);
				iCnt += split_string(&str_rcv_usart[iCnt], &spliter_string[0],',',20);
				iCnt = split_string(&str_rcv_usart[iCnt], &spliter_string[0],',',5);
 */
int8_t split_string(char *string, char *splitted_string,char split_sign, uint8_t max_char2copy){
	register uint8_t 	ui_str_pos=0;
	register char 		act_char;
			 int8_t		ret_val;

	/* search for first character==split_sign */
	do{	act_char=string[ui_str_pos++]; }while ((act_char!=0) && (act_char!=split_sign)); 	

	/* set found pos as retval for further action */
	ret_val = ui_str_pos; 

	/* jump to position of match */
	ui_str_pos--;

	/* split string */
	if (act_char==0){ /* nothing found - loop reached EOS */
			ret_val = 0; /* ret_val=0 - nothing found */
	}else{
		/* check how many char shall be copied */
		if (max_char2copy<=ui_str_pos) {
			ret_val=-1; /* to many signs to copy */
		}else{
			splitted_string[ui_str_pos]=0; /* set EOS */
			do {  ui_str_pos--; splitted_string[ui_str_pos]=string[ui_str_pos]; }while(0<ui_str_pos);
		}/* else */
	}/* else */
	return (ret_val);
}/* int8_t split_string */


/**
 	@brief		copy string A into string B and checks whether given borders

 	@param		*string_source 	- string to copy
	@param		*string_dest	- string to copy into
	@param		max_no_char		- maximum number of characters to copy (limit of string_dest)

 	@return		-1	-	string could not be copied completly
 	@return		 0	- 	string copied
 	
 	@details    This function copys the content of the string "source" into the string "dest".
				Doing this it checkes whether the amount of copied characters is less than 
				the given amount in
				--- Testcode
				char str_rcv[10]="";
				sprintf(str_rcv_usart,"Test");
				iCnt=string_copy(&str_rcv_usart[0], &str_rcv[0], 10);
				sprintf(str_rcv_usart," 23456789");
				iCnt=string_copy(&str_rcv_usart[0], &str_rcv[0], 10);
				sprintf(str_rcv_usart,"123456789AB");
				iCnt=string_copy(&str_rcv_usart[0], &str_rcv[0], 10);
 */
int8_t string_copy(char *string_source, char *string_dest, uint8_t ui_max_char) {
	register uint8_t 	ui_pos=0;
	register char 		act_char;
	do{	
		act_char				=	string_source[ui_pos];
		string_dest[ui_pos++]	=	act_char; 
	}while ( (act_char!=0) && (ui_pos<ui_max_char)); 
	if (act_char!=0) {
		return (-1);
	} else {
		return (0);
	}
}/* int8_t string_copy.... */

#undef  STRING_ADDON_C
